@@ -2,6 +2,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+
+#ifndef _HAVE_SSL
+int main() {
+  fprintf(stderr, "Error: thc-ipv6 was compiled without openssl support, covert_send6d disabled.\n");
+  return -1;
+}
+#else
+
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -12,14 +20,13 @@
 #include <openssl/sha.h>
 #include "thc-ipv6.h"
 
-extern int debug;
 FILE *f;
 BF_KEY bfkey;
 int rawmode = 0, seq = 1, id = 0, num = 0;
 char hash[20] = "", *key = NULL, vec[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-k key] interface file\n\n", prg);
   printf("Options:\n");
   printf("  -k key     decrypt the content with Blowfish-160\n");
@@ -28,13 +35,20 @@ void help(char *prg) {
   exit(-1);
 }
 
-void check_packets(u_char * foo, const struct pcap_pkthdr *header, const unsigned char *data) {
+void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
   int len = header->caplen, rlen, bytes = 0, hlen, end = 0, pos, dlen = 0, done = 0;
   unsigned char *ptr = (unsigned char *) data, rbuf[6000], wbuf[6000];
 
   if (!rawmode) {
-    ptr += 14;
-    len -= 14;
+    if (do_hdr_size) {
+      ptr += do_hdr_size;
+      len -= do_hdr_size;
+      if ((ptr[0] & 240) != 0x60)
+        return;
+    } else {
+      ptr += 14;
+      len -= 14;
+    }
   }
 
   if (len < 58)                 // too short
@@ -142,3 +156,5 @@ int main(int argc, char *argv[]) {
 
   return 0;
 }
+
+#endif
