@@ -1,24 +1,7 @@
-
 /*
- * dnsdict6, (c) 2011 by van Hauser / THC ; most code from:
+ * dnsdict6.h (c) 2012 by van Hauser / THC
  *
- * ** dnsmap - DNS Network Mapper by pagvac
- * ** Copyright (C) 2011 gnucitizen.org
- * **
- * ** This program is free software; you can redistribute it and/or modify
- * ** it under the terms of the GNU General Public License as published by
- * ** the Free Software Foundation; either version 2 of the License, or
- * ** (at your option) any later version.
- * **
- * ** This program is distributed in the hope that it will be useful,
- * ** but WITHOUT ANY WARRANTY; without even the implied warranty of
- * ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * ** GNU General Public License for more details.
- * **
- * ** You should have received a copy of the GNU General Public License
- * ** along with this program; if not, write to the Free Software
- * ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- * */
+ */
 
 // buil-in list of subdomains
 char *sub_small[] = {
@@ -91,6 +74,15 @@ char *sub_medium[] = {
   "imap",
   "ntp",
   "www6",
+  "ip6test",
+  "ipv6test",
+  "6test",
+  "test6",
+  "testip6",
+  "testipv6",
+  "www6test",
+  "test6www",
+  "wwwtest6",
   "ns3",
   "irc",
   "pop3",
@@ -136,6 +128,8 @@ char *sub_medium[] = {
   "git",
   "dns1",
   "vpn",
+  "sslvpn",
+  "ssl-vpn",
   "lg",
   "whois",
   "mail2",
@@ -927,6 +921,8 @@ char *sub_large[] = {
   "git",
   "dns1",
   "vpn",
+  "sslvpn",
+  "ssl-vpn",
   "lg",
   "whois",
   "mail2",
@@ -974,6 +970,15 @@ char *sub_large[] = {
   "m",
   "cache",
   "wap",
+  "ip6test",
+  "ipv6test",
+  "6test",
+  "test6",
+  "testip6",
+  "testipv6",
+  "www6test",
+  "test6www",
+  "wwwtest6",
   "ubuntu",
   "syslog",
   "shell",
@@ -2260,6 +2265,18 @@ char *sub_large[] = {
   "www-6",
   "wwwipv6",
   "wwwv6",
+  "auth0",
+  "auth00",
+  "dns6",
+  "gate1",
+  "gw-ipv6",
+  "ntp3",
+  "rns2",
+  "sec1",
+  "vl-6",
+  "vlan-100",
+  "vlan100",
+  "www-v6v4",
   "dds",
   "vds",
   ""
@@ -2302,6 +2319,15 @@ char *sub_xtreme[] = {
   "9",
   "ILMI",
   "a",
+  "ip6test",
+  "ipv6test",
+  "6test",
+  "test6",
+  "testip6",
+  "testipv6",
+  "www6test",
+  "test6www",
+  "wwwtest6",
   "a.auth-ns",
   "a0",
   "a01",
@@ -5102,6 +5128,8 @@ char *sub_xtreme[] = {
   "voyager",
   "vp",
   "vpn",
+  "sslvpn",
+  "ssl-vpn",
   "vpn0",
   "vpn01",
   "vpn02",
@@ -5426,5 +5454,654 @@ char *sub_xtreme[] = {
   "zx",
   "zy",
   "zz",
+  "2607",
+  "accelerator",
+  "auth0",
+  "auth00",
+  "cacti",
+  "castor",
+  "cpr1",
+  "d1",
+  "dist02",
+  "dlv",
+  "dns-a",
+  "dns-b",
+  "dnsa",
+  "dnsb",
+  "dns6",
+  "edge1",
+  "excalibur",
+  "filesender",
+  "gate1",
+  "gnu",
+  "gw-ipv6",
+  "iana",
+  "in1",
+  "llb1",
+  "lo-6",
+  "moz",
+  "newsfeed2",
+  "no-dns-yet",
+  "ns12",
+  "ns8",
+  "ntp3",
+  "rns2",
+  "sec1",
+  "star-gw",
+  "te2-1",
+  "tge-1-1",
+  "uhura",
+  "vl-6",
+  "vl6-at",
+  "vlan-100",
+  "vlan100",
+  "www-v6v4",
+  ""
+};
+
+
+char *sub_srv_test[] = {
+  "_1password",
+  "_abi-instrument",
+  "_xmpp-server",
+  "_xmpp-client",
+  "_http",
+  "_https",
+  "_sip",
+  "_sips",
+  "_hydra",
+  "_hyperstream",
+  "_iax",
+  "_foo",
+  "_pop3",
+  "_imap",
+  "_abc",
+  "_zzz",
+  ""
+};
+
+char *sub_srv[] = {
+  "_1password",
+  "_abi-instrument",
+  "_accessdata-f2d",
+  "_accessdata-f2w",
+  "_accessone",
+  "_accountedge",
+  "_acrobatsrv",
+  "_actionitems",
+  "_activeraid",
+  "_activeraid-ssl",
+  "_addressbook",
+  "_adisk",
+  "_adobe-vc",
+  "_adpro-setup",
+  "_a-d-sync",
+  "_aecoretech",
+  "_aeroflex",
+  "_afpovertcp",
+  "_airport",
+  "_airprojector",
+  "_airsharing",
+  "_airsharingpro",
+  "_amba-cam",
+  "_amiphd-p2p",
+  "_animobserver",
+  "_animolmd",
+  "_anquetsync",
+  "_appelezvous",
+  "_apple-ausend",
+  "_apple-midi",
+  "_applerdbg",
+  "_apple-sasl",
+  "_appletv",
+  "_appletv-itunes",
+  "_appletv-pair",
+  "_aquamon",
+  "_asr",
+  "_astnotify",
+  "_astralite",
+  "_async",
+  "_atlassianapp",
+  "_auth",
+  "_autodiscover",
+  "_av",
+  "_axis-video",
+  "_b3d-convince",
+  "_babyphone",
+  "_bdsk",
+  "_beacon",
+  "_beamer",
+  "_beatpack",
+  "_beep",
+  "_bfagent",
+  "_bigbangchess",
+  "_bigbangmancala",
+  "_bittorrent",
+  "_blackbook",
+  "_bluevertise",
+  "_bookworm",
+  "_bootps",
+  "_boundaryscan",
+  "_bousg",
+  "_bri",
+  "_bsqdea",
+  "_busycal",
+  "_caltalk",
+  "_cardsend",
+  "_cctv",
+  "_cheat",
+  "_chess",
+  "_chfts",
+  "_chili",
+  "_cip4discovery",
+  "_clipboard",
+  "_clique",
+  "_clscts",
+  "_collection",
+  "_com-ocs-es-mcc",
+  "_contactserver",
+  "_corroboree",
+  "_cpnotebook2",
+  "_cvspserver",
+  "_cw-codetap",
+  "_cw-dpitap",
+  "_cw-oncetap",
+  "_cw-powertap",
+  "_cytv",
+  "_daap",
+  "_dacp",
+  "_dancepartner",
+  "_dataturbine",
+  "_device-info",
+  "_difi",
+  "_disconnect",
+  "_distcc",
+  "_dist-opencl",
+  "_ditrios",
+  "_divelogsync",
+  "_dltimesync",
+  "_dns-llq",
+  "_dns-sd",
+  "_dns-update",
+  "_domain",
+  "_dop",
+  "_dossier",
+  "_dpap",
+  "_dropcopy",
+  "_dsl-sync",
+  "_dtrmtdesktop",
+  "_dvbservdsc",
+  "_dxtgsync",
+  "_ea-dttx-poker",
+  "_earphoria",
+  "_eb-amuzi",
+  "_ebms",
+  "_ebreg",
+  "_ecbyesfsgksc",
+  "_ecms",
+  "_edcp",
+  "_egistix",
+  "_eheap",
+  "_embrace",
+  "_ep",
+  "_eppc",
+  "_erp-scale",
+  "_esp",
+  "_eucalyptus",
+  "_eventserver",
+  "_evs-notif",
+  "_ewalletsync",
+  "_example",
+  "_exb",
+  "_exec",
+  "_extensissn",
+  "_eyetvsn",
+  "_facespan",
+  "_fairview",
+  "_faxstfx",
+  "_feed-sharing",
+  "_firetask",
+  "_fish",
+  "_fix",
+  "_fjork",
+  "_fl-purr",
+  "_fmpro-internal",
+  "_fmserver-admin",
+  "_fontagentnode",
+  "_foxtrot-serv",
+  "_foxtrot-start",
+  "_frameforge-lic",
+  "_freehand",
+  "_frog",
+  "_ftp",
+  "_ftpcroco",
+  "_fv-cert",
+  "_fv-key",
+  "_fv-time",
+  "_garagepad",
+  "_gbs-smp",
+  "_gbs-stp",
+  "_gforce-ssmp",
+  "_glasspad",
+  "_glasspadserver",
+  "_glrdrvmon",
+  "_gpnp",
+  "_grillezvous",
+  "_growl",
+  "_guid",
+  "_h323",
+  "_h323cs",
+  "_h323ls",
+  "_helix",
+  "_help",
+  "_hg",
+  "_hinz",
+  "_hmcp",
+  "_homeauto",
+  "_home-sharing",
+  "_honeywell-vid",
+  "_hotwayd",
+  "_howdy",
+  "_hpr-bldlnx",
+  "_hpr-bldwin",
+  "_hpr-db",
+  "_hpr-rep",
+  "_hpr-toollnx",
+  "_hpr-toolwin",
+  "_hpr-tstlnx",
+  "_hpr-tstwin",
+  "_hs-off",
+  "_htsp",
+  "_http",
+  "_https",
+  "_hydra",
+  "_hyperstream",
+  "_iax",
+  "_ibiz",
+  "_ican",
+  "_ica-networking",
+  "_ichalkboard",
+  "_ichat",
+  "_iconquer",
+  "_idata",
+  "_idcws",
+  "_idsync",
+  "_ifolder",
+  "_ihouse",
+  "_ii-drills",
+  "_ii-konane",
+  "_ilynx",
+  "_imap",
+  "_imaps",
+  "_imidi",
+  "_indigo-dvr",
+  "_inova-ontrack",
+  "_ipbroadcaster",
+  "_ipp",
+  "_ipspeaker",
+  "_irelay",
+  "_irmc",
+  "_iscsi",
+  "_ishare",
+  "_isparx",
+  "_ispq-vc",
+  "_isticky",
+  "_istorm",
+  "_itis-device",
+  "_itsrc",
+  "_ivef",
+  "_iwork",
+  "_jcan",
+  "_jabber",
+  "_jabber-conference",
+  "_jeditx",
+  "_jini",
+  "_jtag",
+  "_kerberos",
+  "_kerberos-adm",
+  "_kerberos-iv",
+  "_kerberos-master",
+  "_kpasswd",
+  "_ktp",
+  "_labyrinth",
+  "_lan2p",
+  "_lanrevagent",
+  "_lanrevserver",
+  "_lapse",
+  "_ldap",
+  "_leaf",
+  "_lexicon",
+  "_liaison",
+  "_library",
+  "_llrp",
+  "_llrp-secure",
+  "_lobby",
+  "_logicnode",
+  "_login",
+  "_lonbridge",
+  "_lontalk",
+  "_lonworks",
+  "_lsys-appserver",
+  "_lsys-camera",
+  "_lsys-ezcfg",
+  "_lsys-oamp",
+  "_lux-dtp",
+  "_lxi",
+  "_lyrics",
+  "_macfoh",
+  "_macfoh-admin",
+  "_macfoh-audio",
+  "_macfoh-data",
+  "_macfoh-db",
+  "_macfoh-events",
+  "_macfoh-remote",
+  "_macminder",
+  "_maestro",
+  "_magicdice",
+  "_mandos",
+  "_matrix",
+  "_mbconsumer",
+  "_mbproducer",
+  "_mbserver",
+  "_mconnect",
+  "_mcrcp",
+  "_mediaboard1",
+  "_mesamis",
+  "_mimer",
+  "_mi-raysat",
+  "_modolansrv",
+  "_moneysync",
+  "_moneyworks",
+  "_moodring",
+  "_mother",
+  "_movieslate",
+  "_mp3sushi",
+  "_mqtt",
+  "_mslingshot",
+  "_mttp",
+  "_mumble",
+  "_musicmachine",
+  "_mxim-art2",
+  "_mxim-ice",
+  "_mxs",
+  "_mysync",
+  "_ncbroadcast",
+  "_ncdirect",
+  "_ncsyncserver",
+  "_neoriders",
+  "_net2display",
+  "_net-assistant",
+  "_netrestore",
+  "_newton-dock",
+  "_nfs",
+  "_nicname",
+  "_nssocketport",
+  "_ntlx-arch",
+  "_ntlx-ent",
+  "_ntlx-video",
+  "_ntp",
+  "_ntx",
+  "_obf",
+  "_objective",
+  "_oce",
+  "_odabsharing",
+  "_odisk",
+  "_od-master",
+  "_officetime-sync",
+  "_ofocus-conf",
+  "_ofocus-sync",
+  "_olpc-activity1",
+  "_oma-bcast-sg",
+  "_omni-bookmark",
+  "_omni-live",
+  "_openbase",
+  "_opencu",
+  "_oprofile",
+  "_oscit",
+  "_ovready",
+  "_owhttpd",
+  "_owserver",
+  "_p2pchat",
+  "_panoply",
+  "_parabay-p2p",
+  "_parentcontrol",
+  "_parliant",
+  "_passwordwallet",
+  "_pcast",
+  "_pdl-datastream",
+  "_pgpkey-hkp",
+  "_pgpkey-http",
+  "_pgpkey-https",
+  "_pgpkey-ldap",
+  "_pgpkey-mailto",
+  "_photoparata",
+  "_pictua",
+  "_piesync",
+  "_piu",
+  "_poch",
+  "_pokeeye",
+  "_pop3",
+  "_pop3s",
+  "_postgresql",
+  "_powereasy-erp",
+  "_powereasy-pos",
+  "_pplayer-ctrl",
+  "_presence",
+  "_print-caps",
+  "_printer",
+  "_printer._sub._http",
+  "_profilemac",
+  "_prolog",
+  "_protonet",
+  "_psap",
+  "_psia",
+  "_ptnetprosrv2",
+  "_ptp",
+  "_ptp-req",
+  "_puzzle",
+  "_qbox",
+  "_qttp",
+  "_quinn",
+  "_radioepg",
+  "_radiotag",
+  "_radiovis",
+  "_rakket",
+  "_raop",
+  "_rbr",
+  "_rce",
+  "_rdp",
+  "_realplayfavs",
+  "_recipe",
+  "_remote",
+  "_remoteburn",
+  "_renderpipe",
+  "_rendezvouspong",
+  "_renkara-sync",
+  "_resacommunity",
+  "_resol-vbus",
+  "_retrospect",
+  "_rfb",
+  "_rfbc",
+  "_rfid",
+  "_riousbprint",
+  "_roku-rcp",
+  "_rql",
+  "_rsmp-server",
+  "_rsync",
+  "_rtsp",
+  "_rubygems",
+  "_safarimenu",
+  "_sallingbridge",
+  "_sallingclicker",
+  "_salutafugijms",
+  "_sandvox",
+  "_scanner",
+  "_sc-golf",
+  "_schick",
+  "_scone",
+  "_scpi-raw",
+  "_scpi-telnet",
+  "_sdsharing",
+  "_see",
+  "_seeCard",
+  "_senteo-http",
+  "_sentillion-vlc",
+  "_sentillion-vlt",
+  "_sepvsync",
+  "_serendipd",
+  "_servereye",
+  "_servermgr",
+  "_services",
+  "_sessionfs",
+  "_sflow",
+  "_sftp-ssh",
+  "_sge-exec",
+  "_sge-qmaster",
+  "_shell",
+  "_shifter",
+  "_shipsgm",
+  "_shipsinvit",
+  "_shoppersync",
+  "_shoutcast",
+  "_simmon",
+  "_simusoftpong",
+  "_sip",
+  "_sips",
+  "_sipuri",
+  "_sironaxray",
+  "_skype",
+  "_sleep-proxy",
+  "_slimcli",
+  "_slimhttp",
+  "_smartenergy",
+  "_smb",
+  "_sms",
+  "_soap",
+  "_socketcloud",
+  "_souschef",
+  "_sox",
+  "_sparechange",
+  "_sparql",
+  "_spearcat",
+  "_spike",
+  "_spincrisis",
+  "_splashsync",
+  "_spl-itunes",
+  "_spr-itunes",
+  "_ssh",
+  "_ssscreenshare",
+  "_stanza",
+  "_stickynotes",
+  "_strateges",
+  "_submission",
+  "_supple",
+  "_surveillus",
+  "_svn",
+  "_swcards",
+  "_switcher",
+  "_swordfish",
+  "_sxqdea",
+  "_sybase-tds",
+  "_syncopation",
+  "_syncqdea",
+  "_synergy",
+  "_synksharing",
+  "_taccounting",
+  "_tango",
+  "_tapinoma-ecs",
+  "_taskcoachsync",
+  "_tbricks",
+  "_tcode",
+  "_tcu",
+  "_teamlist",
+  "_te-faxserver",
+  "_teleport",
+  "_telnet",
+  "_tera-fsmgr",
+  "_tera-mp",
+  "_tf-redeye",
+  "_tftp",
+  "_thumbwrestling",
+  "_ticonnectmgr",
+  "_timbuktu",
+  "_tinavigator",
+  "_tivo-hme",
+  "_tivo-music",
+  "_tivo-photos",
+  "_tivo-remote",
+  "_tivo-videos",
+  "_todogwa",
+  "_tomboy",
+  "_toothpicserver",
+  "_touch-able",
+  "_touch-remote",
+  "_tri-vis-client",
+  "_tri-vis-server",
+  "_tryst",
+  "_tt4inarow",
+  "_ttcheckers",
+  "_ttp4daemon",
+  "_tunage",
+  "_tuneranger",
+  "_ubertragen",
+  "_uddi",
+  "_uddi-inq",
+  "_uddi-pub",
+  "_uddi-sec",
+  "_uddi-sub",
+  "_upnp",
+  "_urlbookmark",
+  "_uswi",
+  "_utest",
+  "_uwsgi",
+  "_ve-decoder",
+  "_ve-encoder",
+  "_ve-recorder",
+  "_visel",
+  "_volley",
+  "_vos",
+  "_vue4rendercow",
+  "_vxi-11",
+  "_walkietalkie",
+  "_webdav",
+  "_webdavs",
+  "_webissync",
+  "_wedraw",
+  "_we-jell",
+  "_whamb",
+  "_whistler",
+  "_wired",
+  "_witap",
+  "_witapvoice",
+  "_wkgrpsvr",
+  "_workgroup",
+  "_workstation",
+  "_wormhole",
+  "_writietalkie",
+  "_ws",
+  "_wtc-heleos",
+  "_wtc-qels",
+  "_wtc-rex",
+  "_wtc-viscostar",
+  "_wtc-wpr",
+  "_wwdcpic",
+  "_xcodedistcc",
+  "_xgate-rmi",
+  "_xgrid",
+  "_xmms2",
+  "_xmp",
+  "_xmpp-client",
+  "_xmpp-server",
+  "_x-on",
+  "_x-plane9",
+  "_xsanclient",
+  "_xsanserver",
+  "_xsansystem",
+  "_xserveraid",
+  "_xsync",
+  "_xtimelicence",
+  "_xtshapro",
+  "_xul-http",
+  "_yakumo",
   ""
 };
