@@ -10,35 +10,41 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
 int rawmode = 0;
 char *multicast6 = NULL;
 int empty = 0;
 
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] [-l] interface add|delete|query [multicast-address [target-address [ttl [own-ip [own-mac-address [destination-mac-address]]]]]]\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s [-l] interface add|delete|query [multicast-address [target-address [ttl [own-ip [own-mac-address [destination-mac-address]]]]]]\n\n", prg);
   printf("Ad(d)vertise or delete yourself - or anyone you want - in a multicast group of your choice\n");
   printf("Query ask on the network who is listening to multicast addresses\n");
-  printf("Use -l to loop and send (in 5s intervals) until Control-C is pressed.\nUse -r to use raw mode.\n\n");
+  printf("Use -l to loop and send (in 5s intervals) until Control-C is pressed.\n");
+//  printf("Use -r to use raw mode.\n\n");
   exit(-1);
 }
 
-void check_packets(u_char * foo, const struct pcap_pkthdr *header, const unsigned char *data) {
-  unsigned char *ptr = (unsigned char *) data;
+void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
+  unsigned char *ptr = (unsigned char *) data, len = header->caplen - 14;
 
   if (rawmode == 0)
     ptr += 14;
+  if (do_hdr_size) {
+    ptr += (do_hdr_size - 14);
+    len -= (do_hdr_size - 14);
+    if ((ptr[0] & 240) != 0x60)
+      return;
+  }
   if (debug)
-    thc_dump_data(ptr, header->caplen - 14 + 14 * rawmode, "Received Packet");
-  if (ptr[6] == 0 && ptr[40] == 0x3a && ptr[41] == 0 && ptr[42] == 5 && ptr[48] == ICMP6_MLD_REPORT && header->caplen - 14 + 14 * rawmode >= 72)
+    thc_dump_data(ptr, len, "Received Packet");
+  if (ptr[6] == 0 && ptr[40] == 0x3a && ptr[41] == 0 && ptr[42] == 5 && ptr[48] == ICMP6_MLD_REPORT && len >= 72)
     if (empty == 1 || memcmp(multicast6, ptr + 56, 16) == 0)
       printf("MLD Report: %s is listening on %s\n", thc_ipv62notation(ptr + 8), thc_ipv62notation(ptr + 56));
 }
 
 int main(int argc, char *argv[]) {
   unsigned char *pkt1 = NULL, buf[24];
-  unsigned char *dst6 = NULL, *src6 = NULL, srcmac[6] = "", *mac = srcmac, dstmac[6] = "", *dmac = dstmac;
+  unsigned char *dst6 = NULL, *src6 = NULL, srcmac[16] = "", *mac = srcmac, dstmac[16] = "", *dmac = dstmac;
   int pkt1_len = 0, i = 0, j;
   char *interface, string[64] = "ip6 and not udp and not tcp";
   int ttl = 1, mode = 0, wait = 0, loop = 0;
@@ -84,7 +90,11 @@ int main(int argc, char *argv[]) {
     multicast6 = thc_resolve6("::");
     empty = 1;
   } else {
-    multicast6 = thc_resolve6(argv[3]);
+    if ((multicast6 = thc_resolve6(argv[3])) == NULL) {
+      fprintf(stderr, "Error: %s does not resolve to a valid IPv6 address\n", argv[3]);
+      exit(-1);
+    }
+
     for (j = 0; j < 16; j++)
       i += multicast6[j];
     if (i == 0)
