@@ -22,28 +22,27 @@
  */
 
 char fuzztype_ether[] = "..............";
-char fuzztype_ip6[] = ".FWWXBXX..............XX..............";
+char fuzztype_ip6[] = "XF..WXBXX..............XX..............";
 char fuzztype_ip6no[] = "........................................";
 char fuzztype_icmp6[] = "XX..";
 char fuzztype_icmp6no[] = "....";
 char fuzztype_icmp6ping[] = "";
-char fuzztype_icmp6ra[] = "BXWWWWWXXWWFFXFWWWWXX............XXFX......FXBFWWXX............XXFXWWW";
+char fuzztype_icmp6ra[] = "........XXWWWXXXX........XX.............XXXX....XXXXXBB..XX.............XXXBB....XX.............XXXBB....X...X....XX.....X......XXXXXXXXX";
 char fuzztype_icmp6ns[] = "XX..XX............XXXX";
 char fuzztype_icmp6na[] = "FFFFXX............XXXX";
+char fuzztype_icmp6nq[] = "FXFX........XX............XX";
 char fuzztype_icmp6mld[] = "WWXXXX............XX";
-char fuzztype_icmp6mld2rep[] =
-  "WFFXFFF................................................XFFF................................................XFFF................................................";
+char fuzztype_icmp6mld2rep[] = "WFFXFFF................................................XFFF................................................XFFF................................................";
 char fuzztype_icmp6mld2que[] = "WWXXXX............XXFWFF";
+char fuzztype_tcp[] = "WWWWWWXXWWWXXWXXXXXXWWWWXX"; 
 
 unsigned char flags[] = { 0, 1, 2, 4, 8, 16, 32, 64, 128, 254, 255 };   // 11
 unsigned char bytes[] = { 0, 1, 254, 255 };     // 4
-
 unsigned short int words[] = { 0x0000, 0x0001, 0x00fe, 0x00ff, 0x0100, 0x0101,
   0x01fe, 0x01ff, 0xfe00, 0xfe01, 0xfefe, 0xfeff, 0xff00, 0xff01, 0xfffe,
   0xffff
 };                              // 16
 unsigned int xors[] = { 0, 0xffffffff };        // 2
-
 unsigned char extends[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
   15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
   34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
@@ -61,7 +60,6 @@ unsigned char extends[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
   228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242,
   243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
 };                              // 256
-
 unsigned int dwords[] = { 0x00000000, 0x00000001, 0x000000fe, 0x000000ff,
   0x00000100, 0x00000101, 0x000001fe, 0x000001ff,
   0x0000fe00, 0x0000fe01, 0x0000fefe, 0x0000feff,
@@ -137,24 +135,27 @@ unsigned int dwords[] = { 0x00000000, 0x00000001, 0x000000fe, 0x000000ff,
 #define NEVER 2000000000
 #define TEST_MAX (NEVER - 1)
 
-#define DO_PING     1
-#define DO_NEIGHSOL 2
-#define DO_NEIGHADV 3
-#define DO_RA       4
+#define DO_NONE      0
+#define DO_PING      1
+#define DO_NEIGHSOL  2
+#define DO_NEIGHADV  3
+#define DO_RA        4
+#define DO_NODEQUERY 5
+#define DO_TCP       6
 #define DO_MLD_REP  131
 #define DO_MLD_DONE  132
 #define DO_MLD_QUERY  130
 #define DO_MLD2_QUERY  256
 #define DO_MLD2_REPORT 143
 
-
-extern int debug;
+int port = -1;
 
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] [-x] [-t number | -T number] [-p number] [-IFSDHRJ] [-1|-2|-3|-4|-5|-6|-7] interface unicast-or-multicast-address [address-in-data-pkt]\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s [-x] [-t number | -T number] [-p number] [-IFSDHRJ] [-X|-1|-2|-3|-4|-5|-6|-7|-8|-9|-0 port] interface unicast-or-multicast-address [address-in-data-pkt]\n\n", prg);
   printf("Fuzzes an icmp6 packet\n");
   printf("Options:\n");
+  printf(" -X         do not add any ICMP/TCP header (tranport laye)\n");
   printf(" -1         fuzz ICMP6 echo request (default)\n");
   printf(" -2         fuzz ICMP6 neighbor solicitation\n");
   printf(" -3         fuzz ICMP6 neighbor advertisement\n");
@@ -164,39 +165,43 @@ void help(char *prg) {
   printf(" -7         fuzz multicast listener query packet\n");
   printf(" -8         fuzz multicast listener v2 report packet\n");
   printf(" -9         fuzz multicast listener v2 query packet\n");
+  printf(" -0         fuzz node query packet\n");
+  printf(" -s port    fuzz TCP-SYN packet against port\n");       
   printf(" -x         tries all 256 values for flag and byte types\n");
   printf(" -t number  continue from test no. number\n");
   printf(" -T number  only performs test no. number\n");
   printf(" -p number  perform an alive check every number of tests (default: none)\n");
+  printf(" -a         do not perform initial and final alive test\n");
   printf(" -n number  how many times to send each packet (default: 1)\n");
-  printf(" -I         fuzz the IP + ICMP header too\n");
+  printf(" -I         fuzz the IP header too\n");
   printf(" -F         add one-shot fragmentation, and fuzz it too (for 1)\n");
   printf(" -S         add source-routing, and fuzz it too (for 1)\n");
   printf(" -D         add destination header, and fuzz it too (for 1)\n");
   printf(" -H         add hop-by-hop header, and fuzz it too (for 1 and 5-9)\n");
   printf(" -R         add router alert header, and fuzz it too (for 5-9 and all)\n");
   printf(" -J         add jumbo packet header, and fuzz it too (for 1)\n");
-  printf("Use -r to use raw mode.\n");
-  printf("You can only define one of -1 ... -7, defaults to -1.\n\n");
+//  printf("Use -r to use raw mode.\n");
+  printf("You can only define one of -0 ... -9 and -s, defaults to -1.\n");
+  printf("Returns -1 on error, 0 on tests done and targt alive or 1 on target crash.\n");
   exit(-1);
 }
 
-void ignoreit(u_char * foo, const struct pcap_pkthdr *header, const unsigned char *data) {
+void ignoreit(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
   return;
 }
 
-int check_alive(pcap_t * p, char *interface, unsigned char *src, unsigned char *dst) {
+int check_alive(pcap_t * p, char *interface, unsigned char *srcmac, unsigned char *dstmac, unsigned char *src, unsigned char *dst) {
   int ret = -2;
   time_t t;
 
   while (thc_pcap_check(p, (char *) ignoreit, NULL) > 0);
-  thc_ping6(interface, src, dst, 16, 1);
+  thc_ping26(interface, srcmac, dstmac, src, dst, 16, 1);
   t = time(NULL);
   while (ret < 0) {
     if (thc_pcap_check(p, (char *) ignoreit, NULL) > 0)
       ret = 1;
     if (time(NULL) > t + 1 && ret == -2) {
-      thc_ping6(interface, src, dst, 16, 1);
+      thc_ping26(interface, srcmac, dstmac, src, dst, 16, 1);
       ret = -1;
     }
     if (time(NULL) > t + 4 && ret < 0)
@@ -208,12 +213,12 @@ int check_alive(pcap_t * p, char *interface, unsigned char *src, unsigned char *
 
 int main(int argc, char *argv[]) {
   unsigned char *pkt = NULL, *pkt_bak, *mcast6, *someaddr6 = NULL;
-  unsigned char *dst6, *src6 = NULL, *mac = NULL, *routers[2], string[64] = "icmp6 and dst ";
-  int rawmode = 0, test_start = 0, fragment = 0, alert = 0, sroute = 0;
+  unsigned char *dst6, *src6 = NULL, *mac = NULL, *routers[2], string[64] = "ip6 and dst ";
+  int test_start = 0, fragment = 0, alert = 0, sroute = 0;
   int do_type = DO_PING, do_alive = 1, hopbyhop = 0, destination = 0, jumbo = 0;
   int pkt_len = 0, offset = 0, test_current = 0, i, j, k, do_fuzz = 1, test_ptr = 0;
   int test_end = TEST_MAX, ping = NEVER, frag_offset = 0, header = 0, no_send = 1;
-  int test_pos = 0, test_cnt = 0, do_it, extend = 0, mtu = 1500, size = 64, wait = 0;
+  int test_pos = 0, test_cnt = 0, do_it, extend = 0, mtu = 1500, size = 64, wait = 0, off2 = 14;
   char *interface, fuzzbuf[256], *srcmac, *dns, *route6, *real_dst6 = NULL;
   unsigned char buf[256], buf2[100], buf3[16];
   unsigned short int *sip;
@@ -223,8 +228,15 @@ int main(int argc, char *argv[]) {
   if (argc < 3 || strncmp(argv[1], "-h", 2) == 0)
     help(argv[0]);
 
-  while ((i = getopt(argc, argv, "123456789rxt:T:p:FSDHRIJan:")) >= 0) {
+  while ((i = getopt(argc, argv, "s:0123456789Xxt:T:p:FSDHRIJan:")) >= 0) {
     switch (i) {
+    case 's':
+      do_type = DO_TCP;
+      port = atoi(optarg);
+      break;
+    case '0':
+      do_type = DO_NODEQUERY;
+      break;
     case '1':
       do_type = DO_PING;
       break;
@@ -254,11 +266,8 @@ int main(int argc, char *argv[]) {
       do_type = DO_MLD2_QUERY;
       wait = 0xff0000;
       break;
-    case 'r':
-      thc_ipv6_rawmode(1);
-      rawmode = 1;
-      test_pos -= 14;
-      offset -= 14;
+    case 'X':
+      do_type = DO_NONE;
       break;
     case 't':
       test_start = atoi(optarg);
@@ -306,7 +315,7 @@ int main(int argc, char *argv[]) {
     exit(-1);
   }
   interface = argv[optind];
-  if (rawmode == 0 && (srcmac = thc_get_own_mac(interface)) == NULL) {
+  if ((srcmac = thc_get_own_mac(interface)) == NULL) {
     fprintf(stderr, "ERROR: %s is not a valid interface which has a MAC, use raw mode?\n", interface);
     exit(-1);
   }
@@ -314,8 +323,17 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "ERROR: -n number must be between one and 2 billion\n");
     exit(-1);
   }
-  if (do_type != DO_PING) {
-    mcast6 = thc_resolve6(argv[optind + 1]);
+  if (do_hdr_size) {
+    test_pos -= do_hdr_size;
+    offset -= do_hdr_size;
+    off2 = do_hdr_size;
+  }
+  if (do_type != DO_PING && do_type != DO_TCP && do_type != DO_NONE) {
+    if ((mcast6 = thc_resolve6(argv[optind + 1])) == NULL) {
+      fprintf(stderr, "Error: %s does not resolve to a valid IPv6 address\n", argv[optind + 1]);
+      exit(-1);
+    }
+
     if (do_type == DO_NEIGHSOL) {
       dst6 = thc_resolve6("ff02::0001:ff00:0000");
       memcpy(dst6 + 13, mcast6 + 13, 3);
@@ -325,7 +343,11 @@ int main(int argc, char *argv[]) {
     dst6 = thc_resolve6(argv[optind + 1]);
   }
   if (argv[optind + 1] != NULL)
-    real_dst6 = thc_resolve6(argv[optind + 1]);
+    if ((real_dst6 = thc_resolve6(argv[optind + 1])) == NULL) {
+      fprintf(stderr, "Error: %s does not resolve to a valid IPv6 address\n", argv[optind + 1]);
+      exit(-1);
+    }
+
 
   if (interface == NULL || argv[optind + 1] == NULL) {
     printf("Error: interface and target-ipv6-address are mandatory command line options\n");
@@ -345,10 +367,8 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "ERROR: Can not resolve mac address for %s\n", argv[2]);
     exit(-1);
   }
-  if (rawmode)
-    srcmac = mac;
 
-  if (do_type == DO_PING)
+  if (do_type == DO_PING || do_type == DO_TCP || do_type == DO_NONE)
     src6 = thc_get_own_ipv6(interface, dst6, PREFER_GLOBAL);
   else
     src6 = thc_get_own_ipv6(interface, dst6, PREFER_LINK);
@@ -363,16 +383,18 @@ int main(int argc, char *argv[]) {
   setvbuf(stdout, NULL, _IONBF, 0);
   memset(buf, 0, sizeof(buf));
   memset(buf2, 0, sizeof(buf2));
-  dns = thc_resolve6("FF02::FB");
+  dns = thc_resolve6("ff02::fb");
   route6 = thc_resolve6("2a01::");
 
   if ((p = thc_pcap_init(interface, string)) == NULL) {
     fprintf(stderr, "Error: could not capture on interface %s with string %s\n", interface, string);
     exit(-1);
   }
+  if (real_dst6 != NULL && real_dst6[0] == 0xff)
+    do_alive = 0;
   // ping before to check if it works
   if (do_alive)
-    if (check_alive(p, interface, src6, real_dst6) == 0) {
+    if (check_alive(p, interface, srcmac, mac, src6, real_dst6) == 0) {
       fprintf(stderr, "Error: target %s is not alive via direct ping6!\n", argv[optind + 1]);
       exit(-1);
     }
@@ -479,16 +501,43 @@ int main(int argc, char *argv[]) {
   }
 
   memset(buf, 0, sizeof(buf));
-  if (header)
+//  if (header)
     strcat(fuzzbuf, fuzztype_icmp6);
-  else
-    strcat(fuzzbuf, fuzztype_icmp6no);
+//  else
+//    strcat(fuzzbuf, fuzztype_icmp6no);
   switch (do_type) {
+  case DO_TCP:
+    // tcp options
+    buf[0] = 2; // max segment size
+    buf[1] = 4;
+    buf[2] = 255;
+    buf[3] = 255;
+    buf[4] = 3; // windows size
+    buf[5] = 3;
+    buf[6] = 62;
+    buf[7] = 1; // padding
+    buf[8] = 8; // timestamp
+    buf[9] = 10;
+    buf[10] = time(NULL) / 16777216;
+    buf[11] = ((time(NULL) / 65536) % 256);
+    buf[12] = ((time(NULL) / 256) % 256);
+    buf[13] = time(NULL) % 256;
+    // 4 bytes ack tstamp 00000000
+    // rest is padding (2 bytes)
+    if (thc_add_tcp(pkt, &pkt_len, 65532, port, test_current, 0, TCP_SYN, 5760, 0, (unsigned char *) buf, 20, (unsigned char *) buf, 20) < 0)
+      return -1;
+    strcat(fuzzbuf, fuzztype_tcp);
+    break;
+
   case DO_PING:
     if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, test_current, (unsigned char *) &buf, 16, 0) < 0)
       return -1;
     strcat(fuzzbuf, fuzztype_icmp6ping);
     break;
+    
+  case DO_NONE:
+    // empty
+    break;
 
   case DO_NEIGHSOL:
     if (someaddr6 != NULL)
@@ -515,6 +564,13 @@ int main(int argc, char *argv[]) {
       return -1;
     strcat(fuzzbuf, fuzztype_icmp6na);
     break;
+    
+  case DO_NODEQUERY:
+    memcpy(buf + 8, real_dst6, 16);
+    if (thc_add_icmp6(pkt, &pkt_len, ICMP6_NODEQUERY, 0, 0x0003003e, (unsigned char *) &buf, 24, 0) < 0)
+      return -1;
+    strcat(fuzzbuf, fuzztype_icmp6nq);
+    break;    
 
   case DO_RA:
     //  buf[3] = 250; // 0-3: reachable timer
@@ -543,17 +599,19 @@ int main(int argc, char *argv[]) {
     buf[i++] = 1;
     buf[i++] = 1;
     memcpy(buf + i, srcmac, 6);
-    i += 6;
+    i += 6; // = 8 == 56
 
     // default route routing option
     buf[i++] = 0x18;            // routing entry option type
     buf[i++] = 0x03;            // length 3 == 24 bytes
-    buf[i++] = 0x00;            // prefix length
+    buf[i++] = 64;            // prefix length /64
     buf[i++] = 0x08;            // priority, highest of course
     i += 2;                     // 52-53 unknown
     buf[i++] = 0x11;            // lifetime, word
     buf[i++] = 0x11;            // lifetime, word
-    i += 16;                    // 56-71 address, all zeros for default
+    buf[i++] = 0x20;
+    buf[i++] = 4;               // 56-71 address, 2004:: for default
+    i += 14; // = 24 == 70               
 
     // dns option
     buf[i++] = 0x19;            // dns option type
@@ -565,7 +623,33 @@ int main(int argc, char *argv[]) {
       memcpy(buf + i, someaddr6, 16);   // dns server
     else
       memcpy(buf + i, dns, 16); // dns server
-    i += 16;
+    i += 16; // = 24 == 94
+
+    // seachlist option
+    buf[i++] = 31;
+    buf[i++] = 4;
+    i += 2;
+    memset(buf + i, 1, 4);      // validity time
+    i += 4;
+    buf[i++] = 3;
+    memcpy(buf + i, "foo", 3);
+    i += 3;
+    buf[i++] = 4;
+    memcpy(buf + i, "corp", 4);
+    i += 5; // + null byte
+    buf[i++] = 5;
+    memcpy(buf + i, "local", 5);
+    i += 5;
+    buf[i++] = 6;
+    memcpy(buf + i, "domain", 6);
+    i += 7; // + null byte
+    // = 32 == 126
+    
+    // flag extension option
+    buf[i++] = 26;
+    buf[i++] = 1;
+    buf[i++] = 0x08;
+    i += 5;
 
     if (thc_add_icmp6(pkt, &pkt_len, ICMP6_ROUTERADV, 0, 0xff080800, buf, i, 0) < 0)
       return -1;
@@ -645,10 +729,10 @@ int main(int argc, char *argv[]) {
       i--;
     hdr->pkt[i] = 0;
     hdr->pkt[i + 1] = 0;
-    hdr->pkt[i + 2] = hdr->pkt[4 + 14 - 14 * rawmode];
-    hdr->pkt[i + 3] = hdr->pkt[5 + 14 - 14 * rawmode];
-    hdr->pkt[4 + 14 - 14 * rawmode] = 0;
-    hdr->pkt[5 + 14 - 14 * rawmode] = 0;
+    hdr->pkt[i + 2] = hdr->pkt[4 + off2];
+    hdr->pkt[i + 3] = hdr->pkt[5 + off2];
+    hdr->pkt[4 + off2] = 0;
+    hdr->pkt[5 + off2] = 0;
   }
 
   if (extend)
@@ -676,7 +760,6 @@ int main(int argc, char *argv[]) {
     do_it = 1;
 
 //printf("[%s] pos[%d]=%c -> %d | pkt[%d] | %d (%d=>%d)| ", /*fuzzbuf*/"", test_ptr, fuzzbuf[test_ptr], test_cnt, test_pos, test_current, test_start, test_end);
-
     switch (fuzzbuf[test_ptr]) {
     case 0:
       break;
@@ -775,21 +858,29 @@ int main(int argc, char *argv[]) {
         if (fragment)
           memcpy(hdr->pkt + frag_offset + 58, (char *) &test_current, 4);
         switch (do_type) {
+        case DO_NONE:
+          // empty
+          break;
         case DO_PING:
           for (i = 0; i < 4 + 1; i++)
             memcpy(hdr->pkt + offset + 58 + i * 4, (char *) &test_current, 4);
           break;
+        case DO_TCP:
+          memcpy(hdr->pkt + offset + 58, (char *) &test_current, 4);
+          break;
         case DO_NEIGHSOL:
         case DO_NEIGHADV:
           break;                // do nothing for these
+        case DO_NODEQUERY:
+          memcpy(hdr->pkt + offset + 66, (char *) &test_current, 4);
+          break;
         case DO_RA:
           memcpy(hdr->pkt + offset + 0x62, (char *) &test_current, 4);  // prefix update
           memcpy(hdr->pkt + offset + 0x7e, hdr->pkt + offset + 0x5e, 16);       // routing update
-          if (rawmode == 0)
-            memcpy(hdr->pkt + 8, (char *) &test_current, 4);    // srcmac update
+          memcpy(hdr->pkt + 8, (char *) &test_current, 4);    // srcmac update
           memcpy(hdr->pkt + offset + 0x72, (char *) &test_current, 4);  // srcmac update
-          memcpy(hdr->pkt + 0x10 + 14 - 14 * rawmode, (char *) &test_current, 4);       // srcip update
-          memcpy(hdr->original_src, hdr->pkt + 8 + 14 - 14 * rawmode, 16);      // srcip update for checksum
+          memcpy(hdr->pkt + 0x10 + off2, (char *) &test_current, 4);       // srcip update
+          memcpy(hdr->original_src, hdr->pkt + 8 + off2, 16);      // srcip update for checksum
           break;
         case DO_MLD_QUERY:
         case DO_MLD_DONE:
@@ -808,27 +899,32 @@ int main(int argc, char *argv[]) {
         }
 
         // regenerate checksum
-        if (do_type == do_type) {       // maybe for later non-icmp stuff
+        if (do_type != DO_TCP && do_type != DO_NONE) {       // maybe for later non-icmp stuff
           hdr->pkt[offset + 56] = 0;
           hdr->pkt[offset + 57] = 0;
           i = checksum_pseudo_header(hdr->original_src, hdr->final_dst, NXT_ICMP6, &hdr->pkt[offset + 54], hdr->pkt_len - offset - 54);
           hdr->pkt[offset + 56] = i / 256;
           hdr->pkt[offset + 57] = i % 256;
+        } else { // TCP
+          hdr->pkt[offset + 70] = 0;
+          hdr->pkt[offset + 71] = 0;
+          i = checksum_pseudo_header(hdr->original_src, hdr->final_dst, NXT_TCP, &hdr->pkt[offset + 54], hdr->pkt_len - offset - 54);
+          hdr->pkt[offset + 70] = i / 256;
+          hdr->pkt[offset + 71] = i % 256;
         }
         // send packet
         for (k = 0; k < no_send; k++) {
-          if (thc_send_pkt(interface, pkt, &pkt_len) < 0)
-            return -1;
-          usleep(1);
+          while(thc_send_pkt(interface, pkt, &pkt_len) < 0)
+            usleep(1);
         }
         printf(".");
         usleep(250);
         // if ping, check ping again
         if ((test_current - test_start) % ping == 0 && test_current != 0 && test_start != test_current)
-          if (check_alive(p, interface, src6, real_dst6) == 0) {
+          if (check_alive(p, interface, srcmac, mac, src6, real_dst6) == 0) {
             i = ((((test_current - test_start) / ping) - 1) * ping) + test_start + 1;
             printf("\nResult: target %s crashed during fuzzing, offending test case no. could be %d to %d\n", argv[optind + 1], i < 0 ? 0 : i, test_current);
-            exit(-1);
+            exit(1);
           }
       }
 //else printf("NOT SENT - NOT IN TEST LIST\n");
@@ -842,9 +938,10 @@ int main(int argc, char *argv[]) {
   printf("\n");
   // ping afterwards to check if it worked
   if (do_alive) {
-    if (check_alive(p, interface, src6, real_dst6) == 0)
+    if (check_alive(p, interface, srcmac, mac, src6, real_dst6) == 0) {
       printf("Result: target %s is NOT alive via direct ping6 - good work! (position: %d)\n", argv[optind + 1], test_pos);
-    else
+      exit(1);
+    } else
       printf("Result: target %s is still alive via direct ping6, better luck next time.\n", argv[optind + 1]);
   }
 
