@@ -1,4 +1,3 @@
-
 /*
  * Test:
  *       1. next header = hopbyhop, but no header
@@ -32,33 +31,39 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
-
 int rawmode = 0;
 int cont = 0;
 
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] interface\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s interface\n\n", prg);
   printf("Identifies test packets by the implementation6 tool, useful to check what\n");
   printf("packets passed a firewall\n");
-  printf("Use -r to use raw mode.\n\n");
+//  printf("Use -r to use raw mode.\n\n");
   exit(-1);
 }
 
-void check_packet(u_char * foo, const struct pcap_pkthdr *header, const unsigned char *data) {
+void check_packet(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
   unsigned char *ipv6hdr;
   unsigned char buf[20];
-  int add = 0;
+  int add = 0, len = header->caplen;
 
   if (debug) {
     printf("DEBUG: packet received\n");
     thc_dump_data((unsigned char *) data, header->caplen, "Received Packet");
   }
 
-  if (rawmode == 0)
-    ipv6hdr = (unsigned char *) (data + 14);
-  else
+  if (rawmode == 0) {
+    if (do_hdr_size) {
+      ipv6hdr = (unsigned char *) (data + do_hdr_size);
+      len -= do_hdr_size;
+      if ((ipv6hdr[0] & 240) != 0x60)
+        return;
+    } else {
+      ipv6hdr = (unsigned char *) (data + 14);
+      len -= 14;
+    }
+  } else
     ipv6hdr = (unsigned char *) data;
 
   if (ipv6hdr[0] >> 4 != 6)
@@ -67,7 +72,7 @@ void check_packet(u_char * foo, const struct pcap_pkthdr *header, const unsigned
   if (ipv6hdr[6] == NXT_ICMP6 && (ipv6hdr[40] == ICMP6_NEIGHBORSOL || ipv6hdr[40] == ICMP6_NEIGHBORADV || ipv6hdr[40] == ICMP6_PARAMPROB || ipv6hdr[40] == ICMP6_TTLEXEED))
     return;
 
-  if (header->caplen > 150) {
+  if (len >= 136) {
     if (ipv6hdr[6] == 0 && ipv6hdr[40] == 0 && ipv6hdr[48] == 0) {
       printf("   Detected (potential) implementation6 test case #%d %s\n", 3, cont == 3 ? "(cont'd)" : "");
       cont = 3;
@@ -82,7 +87,7 @@ void check_packet(u_char * foo, const struct pcap_pkthdr *header, const unsigned
     }
   }
 
-  if (header->caplen > 60) {
+  if (len >= 46) {
     switch (ipv6hdr[6]) {
     case NXT_ICMP6:
       if (1 == 1) {
@@ -155,7 +160,7 @@ int main(int argc, char *argv[]) {
   setvbuf(stdout, NULL, _IONBF, 0);
 
   printf("Waiting for implementation check packets on %s, press Control-C to end.\n", interface);
-  if (thc_pcap_function(interface, string, (char *) check_packet, NULL) < 0) {
+  if (thc_pcap_function(interface, string, (char *) check_packet, 0, NULL) < 0) {
     fprintf(stderr, "Error: could not capture on interface %s with string %s\n", interface, string);
     exit(-1);
   }
