@@ -1,6 +1,5 @@
-
 /*
- * (c) 2011 by van Hauser / THC
+ * (c) 2012 by van Hauser / THC
  *
  * THC IPv6 Attack Library Header Files
  *
@@ -11,9 +10,11 @@
 #define _THC_IPV6_H
 
 #include <pcap.h>
-#include <openssl/rsa.h>
+#ifdef _HAVE_SSL
+  #include <openssl/rsa.h>
+#endif
 
-#define VERSION 	"v1.6"
+#define VERSION 	"v2.1"
 #define AUTHOR 		"van Hauser / THC <vh@thc.org>"
 #define RESOURCE	"www.thc.org"
 
@@ -41,7 +42,9 @@
 #define ICMP6_NEIGHBORADV 136
 #define ICMP6_REDIR       137
 #define ICMP6_INFOREQUEST 139
+#define ICMP6_NODEQUERY   139
 #define ICMP6_INFOREPLY   140
+#define ICMP6_NODEREPLY   140
 #define ICMP6_INVNEIGHBORSOL 141
 #define ICMP6_INVNEIGHBORADV 142
 #define ICMP6_MLD2_REPORT 143
@@ -55,7 +58,6 @@
 #define ICMP6_ROUTERPROXYSOL 154
 #define ICMP6_ROUTERPROXYADV 155
 
-
 #define ICMP6_NEIGHBORADV_ROUTER   0x80000000
 #define ICMP6_NEIGHBORADV_SOLICIT  0x40000000
 #define ICMP6_NEIGHBORADV_OVERRIDE 0x20000000
@@ -64,14 +66,18 @@
 #define PREFER_LINK	32
 #define PREFER_GLOBAL	 0
 
+extern int debug;
+extern int _thc_ipv6_showerrors;
+extern int do_hdr_size;
+
 extern void thc_ipv6_show_errors(int mode);
-extern int thc_pcap_function(char *interface, unsigned char *capture, char *function, char *opt);
-extern pcap_t *thc_pcap_init(char *interface, unsigned char *capture);
+extern int thc_pcap_function(char *interface, char *capture, char *function, int promisc, char *opt);
+extern pcap_t *thc_pcap_init(char *interface, char *capture);
 extern pcap_t *thc_pcap_init_promisc(char *interface, unsigned char *capture);
 extern void thc_ipv6_rawmode(int mode);
 extern int thc_pcap_check(pcap_t * p, char *function, char *opt);
 extern char *thc_pcap_close(pcap_t * p);
-extern char *thc_resolve6(unsigned char *target);
+extern unsigned char *thc_resolve6(char *target);
 extern unsigned char *thc_lookup_ipv6_mac(char *interface, unsigned char *dst);
 extern unsigned char *thc_get_own_mac(char *interface);
 extern int thc_get_mtu(char *interface);
@@ -80,19 +86,19 @@ extern unsigned char *thc_get_multicast_mac(unsigned char *dst);
 extern unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *dst);
 extern unsigned char *thc_inverse_packet(unsigned char *pkt, int pkt_len);
 extern int thc_ping6(char *interface, unsigned char *src, unsigned char *dst, int size, int count);
+extern int thc_ping26(char *interface, unsigned char *srcmac, unsigned char *dstmac, unsigned char *src, unsigned char *dst, int size, int count);
 extern int thc_neighboradv6(char *interface, unsigned char *src, unsigned char *dst, unsigned char *srcmac, unsigned char *dstmac, unsigned int flags, unsigned char *target);
 extern int thc_neighborsol6(char *interface, unsigned char *src, unsigned char *dst, unsigned char *target, unsigned char *srcmac, unsigned char *dstmac);
-extern int thc_routeradv6(char *interface, unsigned char *src, unsigned char *dst, unsigned char *srcmac, unsigned char default_ttl, int managed, unsigned char *prefix,
-                          int prefixlen, int mtu, unsigned int lifetime);
+extern int thc_routeradv6(char *interface, unsigned char *src, unsigned char *dst, unsigned char *srcmac, unsigned char default_ttl, int managed, unsigned char *prefix, int prefixlen, int mtu, unsigned int lifetime);
 extern int thc_routersol6(char *interface, unsigned char *src, unsigned char *dst, unsigned char *srcmac, unsigned char *dstmac);
 extern int thc_toobig6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned int mtu, unsigned char *pkt, int pkt_len);
-extern int thc_paramprob6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned char code, unsigned int pointer, unsigned char *pkt,
-                          int pkt_len);
+extern int thc_paramprob6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned char code, unsigned int pointer, unsigned char *pkt, int pkt_len);
 extern int thc_unreach6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned char code, unsigned char *pkt, int pkt_len);
-extern int thc_redir6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned char *newrouter, unsigned char *newroutermac,
-                      unsigned char *pkt, int pkt_len);
+extern int thc_redir6(char *interface, unsigned char *src, unsigned char *srcmac, unsigned char *dstmac, unsigned char *newrouter, unsigned char *newroutermac, unsigned char *pkt, int pkt_len);
 extern int thc_send_as_fragment6(char *interface, unsigned char *src, unsigned char *dst, unsigned char type, unsigned char *data, int data_len, int frag_len);
-extern char *thc_create_ipv6(char *interface, int prefer, int *pkt_len, unsigned char *src, unsigned char *dst, int ttl, int length, int label, int class, int version);
+extern int thc_send_as_overlapping_first_fragment6(char *interface, unsigned char *src, unsigned char *dst, unsigned char type, unsigned char *data, int data_len, int frag_len, int overlap_spoof_type);
+extern int thc_send_as_overlapping_last_fragment6(char *interface, unsigned char *src, unsigned char *dst, unsigned char type, unsigned char *data, int data_len, int frag_len, int overlap_spoof_type);
+extern unsigned char *thc_create_ipv6(char *interface, int prefer, int *pkt_len, unsigned char *src, unsigned char *dst, int ttl, int length, int label, int class, int version);
 extern int thc_add_hdr_misc(unsigned char *pkt, int *pkt_len, unsigned char type, int len, unsigned char *buf, int buflen);
 extern int thc_add_hdr_route(unsigned char *pkt, int *pkt_len, unsigned char **routers, unsigned char routerptr);
 extern int thc_add_hdr_mobileroute(unsigned char *pkt, int *pkt_len, unsigned char *dst);
@@ -102,8 +108,7 @@ extern int thc_add_hdr_dst(unsigned char *pkt, int *pkt_len, unsigned char *buf,
 extern int thc_add_hdr_hopbyhop(unsigned char *pkt, int *pkt_len, unsigned char *buf, int buflen);
 extern int thc_add_hdr_nonxt(unsigned char *pkt, int *pkt_len, int hdropt);
 extern int thc_add_icmp6(unsigned char *pkt, int *pkt_len, int type, int code, unsigned int flags, unsigned char *data, int data_len, int checksum);
-extern int thc_add_tcp(unsigned char *pkt, int *pkt_len, unsigned short int sport, unsigned short int dport, unsigned int sequence, unsigned int ack, unsigned char flags,
-                       unsigned short int window, unsigned short int urgent, char *option, int option_len, char *data, int data_len);
+extern int thc_add_tcp(unsigned char *pkt, int *pkt_len, unsigned short int sport, unsigned short int dport, unsigned int sequence, unsigned int ack, unsigned char flags, unsigned short int window, unsigned short int urgent, char *option, int option_len, char *data, int data_len);
 extern int thc_add_udp(unsigned char *pkt, int *pkt_len, unsigned short int sport, unsigned short int dport, unsigned int checksum, char *data, int data_len);
 extern int thc_add_data6(unsigned char *pkt, int *pkt_len, unsigned char type, unsigned char *data, int data_len);
 extern int thc_generate_and_send_pkt(char *interface, unsigned char *srcmac, unsigned char *dstmac, unsigned char *pkt, int *pkt_len);
@@ -127,10 +132,8 @@ extern int thc_bind_multicast_to_socket(int s, char *interface, char *src);
 #define DO_CHECKSUM 0xfaf4
 
 #define NXT_IP6 41
-
 #define NXT_INVALID 128
 #define NXT_IGNORE 31
-
 #define NXT_HDR 0
 #define NXT_HOP 0
 #define NXT_HBH 0
@@ -139,15 +142,13 @@ extern int thc_bind_multicast_to_socket(int s, char *interface, char *src);
 #define NXT_NONXT 59
 #define NXT_OPTS 60
 #define NXT_DST 60
-
 #define NXT_ESP 50
 #define NXT_AH 51
-
 #define NXT_MIPV6 135
+#define NXT_PIM 103
 #define NXT_ICMP6 58
 #define NXT_TCP 6
 #define NXT_UDP 17
-
 #define NXT_DATA 255
 
 #define IPV6_FRAME_TYPE 0x86dd
@@ -260,6 +261,8 @@ typedef struct {
 } sockaddr_nl;
 */
 
+#ifdef _HAVE_SSL
+
 typedef struct {
   unsigned char type;
   unsigned char len;
@@ -311,9 +314,11 @@ typedef struct {
   int len;
 } opt_t;
 
-extern thc_key_t *thc_generate_key(int key_len);
-extern thc_cga_hdr *thc_generate_cga(unsigned char *prefix, thc_key_t * key, unsigned char **cga);
+  extern thc_key_t *thc_generate_key(int key_len);
+  extern thc_cga_hdr *thc_generate_cga(unsigned char *prefix, thc_key_t * key, unsigned char **cga);
+
 extern int thc_add_send(unsigned char *pkt, int *pkt_len, int type, int code, unsigned int flags, unsigned char *data, int data_len, thc_cga_hdr * cga_hdr, thc_key_t * key,
                         unsigned char *tag, int checksum);
+#endif
 
 #endif
